#include "vat.hpp"

Vat::Vat(int x, int y, char* string, float* initial_value, COLOR_TYPE color) :
       Icon(x, y, x + VAT_WIDTH, y + VAT_HEIGHT)
{
	current_level = initial_value;
	strcpy(vat_legend, string);
	fluid_x1= x;
	fluid_y1 = y + 1;
	fluid_x2 = x + VAT_WIDTH;
	fluid_y2 = y + VAT_HEIGHT - VAT_LEGEND_HEIGHT;

	fluid_height = 0;
	flow_increment = 0.01;

	border_color = YELLOW;
	fill_color = BLACK;
	fluid_color = color;

	fluid_level = new Box(x1, y1,
		      x2 - x1,
		      VAT_HEIGHT - VAT_LEGEND_HEIGHT,
		      border_color,
		      fill_color);

	title_box = new Box(x1, y2 - VAT_LEGEND_HEIGHT,
		      x2 - x1,
		      VAT_LEGEND_HEIGHT,
		      border_color,
		      border_color);
}

void Vat::Display()
{
	fluid_level->Display();
	title_box->Display();

	// partition off the legend area
	BOX_TYPE title;
	title.x1 = x1;
	title.y1 = y2 - VAT_LEGEND_HEIGHT;
	title.x2 = x2;
	title.y2 = y2;
	Legend title_legend;
	title_legend.Center(&title, BLACK, vat_legend);

	// assume no part of the fluid is currently shown,
	// so the fluid level gets updated
	fluid_height = 0;
	Update();
}

void Vat::Update()
{
	*current_level += flow_increment;

	if (*current_level >= 0.99)
	  flow_increment = -0.01;

	if (*current_level <= 0.01)
	  flow_increment = 0.01;

	int level = Level(*current_level);

	if (level > fluid_height)
	  IncreaseLevel(level);
	else
	  DecreaseLevel(level);
}

void Vat::DecreaseLevel(int level)
{
	// erase excess part of fluid
	BOX_TYPE box;
	box.x1 = fluid_x1 + 1;
	box.y1 = fluid_y1 + level;
	box.x2 = fluid_x2 - 2;
	box.y2 = fluid_y1 + fluid_height;
	GraphicsHandler->FillBox(fill_color, &box);

	// adjust vat fluid coordinate
	fluid_height = level;
}

void Vat::IncreaseLevel(int level)
{
	// add a piece to the fluid
	BOX_TYPE box;
	box.x1 = fluid_x1 + 1;
	box.y1 = fluid_y1 + fluid_height;
	box.x2 = fluid_x2 - 2;
	box.y2 = fluid_y1 + level;
	GraphicsHandler->FillBox(fluid_color, &box);

	// adjust vat fluid coordinate
	fluid_height = level;
}

int Vat::Level(float value)
{
	// convert the setting from range (0..1) to the proper
	// range
	return value * (fluid_y2 - 1 - fluid_y1);
}

Vat::~Vat()
{
	delete fluid_level;
	delete title_box;
}
